@@ -1,4 +1,4 @@
-/* $Id: p4.c,v 1.4 2003/05/15 17:36:28 mikpe Exp $
+/* $Id: p4.c,v 1.5 2003/06/15 22:04:32 mikpe Exp $
  *
  * pipe stdout through 'sort -u' to see:
  * - which ESCRs are usable, and the events they support
@@ -427,7 +427,7 @@ static const struct event events[] = {
     { "IOQ_active_entries", 0x1A, FSB_ESCR1, -1 },
     { "FSB_data_activity", 0x17, FSB_ESCR0, FSB_ESCR1 },
     { "BSQ_allocation", 0x05, BSU_ESCR0, -1 },
-    { "bsq_active_entries", 0x06, BSU_ESCR1, -1 },	/* NOTE: typo in manual */
+    { "bsq_active_entries", 0x06, BSU_ESCR1, -1 },
     { "SSE_input_assist", 0x34, FIRM_ESCR0, FIRM_ESCR1 },
     { "packed_SP_uop", 0x08, FIRM_ESCR0, FIRM_ESCR1 },
     { "packed_DP_uop", 0x0C, FIRM_ESCR0, FIRM_ESCR1 },
@@ -437,7 +437,7 @@ static const struct event events[] = {
     { "128bit_MMX_uop", 0x1A, FIRM_ESCR0, FIRM_ESCR1 },
     { "x87_FP_uop", 0x04, FIRM_ESCR0, FIRM_ESCR1 },
     { "x87_SIMD_moves_uop", 0x2E, FIRM_ESCR0, FIRM_ESCR1 },
-    { "global_power_events", 0x05, FSB_ESCR0, FSB_ESCR1 },
+    { "global_power_events", 0x13, FSB_ESCR0, FSB_ESCR1 },
     { "tc_ms_xfer", 0x05, MS_ESCR0, MS_ESCR1 },
     { "uop_queue_writes", 0x09, MS_ESCR0, MS_ESCR1 },
     { "retired_mispred_branch_type", 0x05, TBPU_ESCR0, TBPU_ESCR1 },
