@@ -1,4 +1,4 @@
-/* $Id: virtual.c,v 1.65 2003/03/09 00:28:35 mikpe Exp $
+/* $Id: virtual.c,v 1.68 2003/06/01 12:10:38 mikpe Exp $
  * Virtual per-process performance counters.
  *
  * Copyright (C) 1999-2003  Mikael Pettersson
@@ -32,9 +32,7 @@ struct vperfctr {
 	atomic_t count;
 	spinlock_t owner_lock;
 	struct task_struct *owner;
-#ifdef CONFIG_SMP
 	unsigned int sampling_timer;
-#endif
 #ifdef CONFIG_PERFCTR_DEBUG
 	unsigned start_smp_id;
 	unsigned suspended;
@@ -182,8 +180,10 @@ static void put_vperfctr(struct vperfctr *perfctr)
  *								*
  ****************************************************************/
 
-/* PRE: perfctr == TASK_VPERFCTR(current) && IS_RUNNING(perfctr)
+/* PRE: IS_RUNNING(perfctr)
  * Suspend the counters.
+ * XXX: When called from switch_to(), perfctr belongs to 'prev'
+ * but current is 'next'. Debug messages will refer to 'next'...
  */
 static inline void vperfctr_suspend(struct vperfctr *perfctr)
 {
@@ -194,10 +194,8 @@ static inline void vperfctr_suspend(struct vperfctr *perfctr)
 
 static inline void vperfctr_reset_sampling_timer(struct vperfctr *perfctr)
 {
-#ifdef CONFIG_SMP
 	/* XXX: base the value on perfctr_info.cpu_khz instead! */
 	perfctr->sampling_timer = HZ/2;
-#endif
 }
 
 /* PRE: perfctr == TASK_VPERFCTR(current) && IS_RUNNING(perfctr)
@@ -323,11 +321,10 @@ void __vperfctr_resume(struct vperfctr *perfctr)
 		vperfctr_resume(perfctr);
 }
 
-#ifdef CONFIG_SMP
 /* Called from update_one_process() [triggered by timer interrupt].
  * PRE: perfctr == TASK_VPERFCTR(current).
  * Sample the counters but do not suspend them.
- * Needed on SMP to avoid precision loss due to multiple counter
+ * Needed to avoid precision loss due to multiple counter
  * wraparounds between resume/suspend for CPU-bound processes.
  * PREEMPT note: called in IRQ context with preemption disabled.
  */
@@ -336,7 +333,6 @@ void __vperfctr_sample(struct vperfctr *perfctr)
 	if( --perfctr->sampling_timer == 0 )
 		vperfctr_sample(perfctr);
 }
-#endif
 
 /****************************************************************
  *								*
@@ -561,11 +557,7 @@ static struct task_struct *get_task_by_proc_pid_inode(struct inode *inode)
 #else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) */
 static struct task_struct *get_task_by_proc_pid_inode(struct inode *inode)
 {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,4)
 	struct task_struct *tsk = PROC_I(inode)->task;
-#else
-	struct task_struct *tsk = inode->u.proc_i.task;
-#endif
 	get_task_struct(tsk);
 	return tsk;
 }
@@ -712,9 +704,7 @@ static void vperfctr_stub_init(void)
 	vperfctr_stub.exit = __vperfctr_exit;
 	vperfctr_stub.suspend = __vperfctr_suspend;
 	vperfctr_stub.resume = __vperfctr_resume;
-#ifdef CONFIG_SMP
 	vperfctr_stub.sample = __vperfctr_sample;
-#endif
 	vperfctr_stub.file_ops = &vperfctr_file_ops;
 	write_unlock(&vperfctr_stub_lock);
 }
