@@ -1,4 +1,4 @@
-/* $Id: x86_64.c,v 1.2 2003/05/21 20:13:47 mikpe Exp $
+/* $Id: x86_64.c,v 1.3 2003/06/15 21:08:09 mikpe Exp $
  * x86_64 performance-monitoring counters driver.
  *
  * Copyright (C) 2003  Mikael Pettersson
@@ -533,53 +533,43 @@ static void perfctr_pm_resume(void)
 	printk("perfctr: PM resume\n");
 }
 
-/* XXX: x86_64 hasn't converted apic & nmi to driver model yet */
-#if 0 && LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,67)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,71)
 
-#include <linux/device.h>
+#include <linux/sysdev.h>
 
-static int perfctr_device_suspend(struct device *dev, u32 state, u32 level)
+static int perfctr_device_suspend(struct sys_device *dev, u32 state)
 {
-	if( level == SUSPEND_POWER_DOWN )
-		perfctr_pm_suspend();
+	perfctr_pm_suspend();
 	return 0;
 }
 
-static int perfctr_device_resume(struct device *dev, u32 level)
+static int perfctr_device_resume(struct sys_device *dev)
 {
-	if( level == RESUME_POWER_ON )
-		perfctr_pm_resume();
+	perfctr_pm_resume();
 	return 0;
 }
 
-static struct device_driver perfctr_driver = {
-	.name		= "perfctr",
-	.bus		= &system_bus_type,
+static struct sysdev_class perfctr_sysclass = {
+	set_kset_name("perfctr"),
 	.resume		= perfctr_device_resume,
 	.suspend	= perfctr_device_suspend,
 };
 
 static struct sys_device device_perfctr = {
-	.name	= "perfctr",
 	.id	= 0,
-	.dev	= {
-		.name	= "perfctr",
-		.driver	= &perfctr_driver,
-	},
+	.cls	= &perfctr_sysclass,
 };
 
 static void __init x86_pm_init(void)
 {
-	if( cpu_has_apic )
-		device_perfctr.dev.parent = &device_lapic.dev;
-	driver_register(&perfctr_driver);
-	sys_device_register(&device_perfctr);
+	if( sysdev_class_register(&perfctr_sysclass) == 0 )
+		sys_device_register(&device_perfctr);
 }
 
 static void __exit x86_pm_exit(void)
 {
 	sys_device_unregister(&device_perfctr);
-	driver_unregister(&perfctr_driver);
+	sysdev_class_unregister(&perfctr_sysclass);
 }
 
 #else	/* 2.4 kernel */
@@ -623,8 +613,7 @@ static inline void x86_pm_exit(void) { }
 
 #ifdef NMI_LOCAL_APIC
 
-/* XXX: x86_64 hasn't converted apic & nmi to driver model yet */
-#if 1 || LINUX_VERSION_CODE < KERNEL_VERSION(2,5,67)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,71)
 static void __init disable_lapic_nmi_watchdog(void)
 {
 #ifdef CONFIG_PM
