@@ -1,4 +1,4 @@
-/* $Id: perfctr.h,v 1.37 2003/03/09 16:43:34 mikpe Exp $
+/* $Id: perfctr.h,v 1.40 2003/05/29 21:26:13 mikpe Exp $
  * x86 Performance-Monitoring Counters driver
  *
  * Copyright (C) 1999-2003  Mikael Pettersson
@@ -27,7 +27,7 @@ struct perfctr_cpu_control {
 		unsigned int pebs_enable;	/* for replay tagging */
 		unsigned int pebs_matrix_vert;	/* for replay tagging */
 	} p4;
-	int ireset[18];			/* <= 0, for i-mode counters */
+	int ireset[18];			/* < 0, for i-mode counters */
 };
 
 struct perfctr_cpu_state {
@@ -110,7 +110,7 @@ extern int perfctr_cpu_init(void);
 extern void perfctr_cpu_exit(void);
 
 /* CPU type name. */
-extern char *perfctr_cpu_name[];
+extern char *perfctr_cpu_name;
 
 /* Hardware reservation. */
 extern const char *perfctr_cpu_reserve(const char *service);
@@ -158,7 +158,7 @@ extern unsigned int perfctr_cpu_identify_overflow(struct perfctr_cpu_state*);
 static inline void perfctr_cpu_set_ihandler(perfctr_ihandler_t x) { }
 #endif
 
-#if defined(CONFIG_SMP) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#if defined(CONFIG_SMP) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,15)
 /* CPUs in `perfctr_cpus_forbidden_mask' must not use the
    performance-monitoring counters. TSC use is unrestricted. */
 extern unsigned long perfctr_cpus_forbidden_mask;
