@@ -1,4 +1,4 @@
-/* $Id: event_set_amd.c,v 1.4 2003/02/28 22:39:08 mikpe Exp $
+/* $Id: event_set_amd.c,v 1.6 2003/05/29 19:13:37 mikpe Exp $
  * Performance counter event descriptions for AMD K7 and K8.
  *
  * Copyright (C) 2003  Mikael Pettersson
@@ -10,6 +10,10 @@
  * AMD Publication #22007
  * Revision E (on AMD Processor Technical Documents CD, Med-12/99-0, 21860F)
  * Revision K (at http://www.amd.com/).
+ *
+ * [K8] "BIOS and Kernel Developer's Guide for AMD Athlon 64 and
+ * AMD Opteron Processors", Chapter 10: "Performance Monitoring".
+ * AMD Publication #26094, Revision 3.00 (at http://www.amd.com).
  */
 #include <stddef.h>	/* for NULL */
 #include "libperfctr.h"
@@ -23,11 +27,8 @@
  * revisions of #22007, and an "unofficial" part which includes events
  * only documented in older revisions of #22007 (specifically Rev. E).
  *
- * The split is motivated partly by recent unofficial K8 information
- * which indicates that (a) most unofficial K7 events are implemented
- * in the K8, but (b) some of them have different unit masks. Therefore,
- * the K8 event list includes only the official K7 events, and gives
- * its own K8 versions of the unofficial K7 events.
+ * All official K7 events are also present in K8, as are most of the
+ * unofficial K7 events.
  */
 
 static const struct perfctr_unit_mask_5 k7_um_moesi = {
@@ -55,7 +56,7 @@ static const struct perfctr_event k7_official_events[] = {
     { 0x45, 0xF, NULL, "L1_DTLB_MISSES_AND_L2_DTLB_HITS",
       "L1 DTLB misses and L2 DTLB hits" },
     { 0x46, 0xF, NULL, "L1_AND_L2_DTLB_MISSES",
-      "L1 and l2 DTLB misses" },
+      "L1 and L2 DTLB misses" },
     { 0x47, 0xF, NULL, "MISALIGNED_DATA_REFERENCES",
       "Misaligned data references" },
     { 0x80, 0xF, NULL, "INSTRUCTION_CACHE_FETCHES",
@@ -98,6 +99,7 @@ static const struct perfctr_event_set k7_official_event_set = {
     .events = k7_official_events,
 };
 
+/* also in K8 */
 static const struct perfctr_unit_mask_7 k7_um_seg_reg = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x3F,
@@ -169,7 +171,7 @@ static const struct perfctr_unit_mask_8 k7_um_L2_requests = {
 };
 
 static const struct perfctr_event k7_unofficial_events[] = {
-    { 0x20, 0xF, UM(k7_um_seg_reg), "SEGMENT_REGISTER_LOADS",
+    { 0x20, 0xF, UM(k7_um_seg_reg), "SEGMENT_REGISTER_LOADS", /* also in K8 */
       "Segment register loads" },
     { 0x21, 0xF, NULL, "STORES_TO_ACTIVE_INSTRUCTION_STREAM", /* also in K8 as SELF_MODIFY_RESYNC */
       "Stores to active instruction stream" },
@@ -183,7 +185,7 @@ static const struct perfctr_event k7_unofficial_events[] = {
       "Single-bit ECC errors detected/corrected" },
     { 0x75, 0xF, UM(k7_um_invalidates), "INTERNAL_CACHE_LINE_INVALIDATES", /* not in K8 */
       "Internal cache line invalidates" },
-    { 0x76, 0xF, NULL, "CYCLES_PROCESSOR_IS_RUNNING", /* not in K8 */
+    { 0x76, 0xF, NULL, "CYCLES_PROCESSOR_IS_RUNNING", /* also in K8, unofficially */
       "Cycles processor is running (not in HLT or STPCLK)" },
     { 0x79, 0xF, UM(k7_um_L2_requests), "L2_REQUESTS", /* not in K8 */
       "L2 requests" },
@@ -249,13 +251,11 @@ const struct perfctr_event_set perfctr_k7_event_set = {
 
 /*
  * AMD K8 events.
- *
- * Unofficial. Based on oprofile/libop/op_events.c.
  */
 
 static const struct perfctr_unit_mask_6 k8_um_fpu_ops = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x3F,
       .nvalues = 6 },
     { { 0x01, "Add pipe ops excluding junk ops" },
       { 0x02, "Multiply pipe ops excluding junk ops" },
@@ -266,35 +266,63 @@ static const struct perfctr_unit_mask_6 k8_um_fpu_ops = {
 };
 
 static const struct perfctr_unit_mask_2 k8_um_ecc = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x03,
       .nvalues = 2 },
     { { 0x01, "Scrubber error" },
       { 0x02, "Piggyback scrubber errors" } }
 };
 
 static const struct perfctr_unit_mask_3 k8_um_prefetch = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
       .nvalues = 3 },
     { { 0x01, "Load" },
       { 0x02, "Store" },
       { 0x04, "NTA" } }
 };
 
+static const struct perfctr_unit_mask_5 k8_um_int_L2_req = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x1F,
+      .nvalues = 5 },
+    { { 0x01, "IC fill" },
+      { 0x02, "DC fill" },
+      { 0x04, "TLB reload" },
+      { 0x08, "Tag snoop request" },
+      { 0x10, "Cancelled request" } }
+};
+
+static const struct perfctr_unit_mask_3 k8_um_fill_req = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
+      .nvalues = 3 },
+    { { 0x01, "IC fill" },
+      { 0x02, "DC fill" },
+      { 0x04, "TLB reload" } }
+};
+
+static const struct perfctr_unit_mask_2 k8_um_fill_L2 = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x03,
+      .nvalues = 2 },
+    { { 0x01, "Dirty L2 victim" },
+      { 0x02, "Victim from L2" } }
+};
+
 static const struct perfctr_unit_mask_4 k8_um_fpu_instr = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x0F,
       .nvalues = 4 },
     { { 0x01, "x87 instructions" },
-      { 0x02, "Combined MMX & 3DNow instructions" },
+      { 0x02, "Combined MMX & 3DNow! instructions" },
       { 0x04, "Combined packed SSE and SSE2 instructions" },
-      { 0x08, "Combined packed scalar SSE and SSE2 instructions" } }
+      { 0x08, "Combined scalar SSE and SSE2 instructions" } }
 };
 
 static const struct perfctr_unit_mask_3 k8_um_fpu_fastpath = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
       .nvalues = 3 },
     { { 0x01, "With low op in position 0" },
       { 0x02, "With low op in position 1" },
@@ -302,8 +330,8 @@ static const struct perfctr_unit_mask_3 k8_um_fpu_fastpath = {
 };
 
 static const struct perfctr_unit_mask_4 k8_um_fpu_exceptions = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x0F,
       .nvalues = 4 },
     { { 0x01, "x87 reclass microfaults" },
       { 0x02, "SSE retype microfaults" },
@@ -312,8 +340,8 @@ static const struct perfctr_unit_mask_4 k8_um_fpu_exceptions = {
 };
 
 static const struct perfctr_unit_mask_3 k8_um_page_access = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
       .nvalues = 3 },
     { { 0x01, "Page hit" },
       { 0x02, "Page miss" },
@@ -321,8 +349,8 @@ static const struct perfctr_unit_mask_3 k8_um_page_access = {
 };
 
 static const struct perfctr_unit_mask_3 k8_um_turnaround = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
       .nvalues = 3 },
     { { 0x01, "DIMM turnaround" },
       { 0x02, "Read to write turnaround" },
@@ -330,8 +358,8 @@ static const struct perfctr_unit_mask_3 k8_um_turnaround = {
 };
 
 static const struct perfctr_unit_mask_4 k8_um_saturation = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x0F,
       .nvalues = 4 },
     { { 0x01, "Memory controller high priority bypass" },
       { 0x02, "Memory controller low priority bypass" },
@@ -340,8 +368,8 @@ static const struct perfctr_unit_mask_4 k8_um_saturation = {
 };
 
 static const struct perfctr_unit_mask_7 k8_um_sized_commands = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x7F,
       .nvalues = 7 },
     { { 0x01, "NonPostWrSzByte" },
       { 0x02, "NonPostWrSzDword" },
@@ -353,8 +381,8 @@ static const struct perfctr_unit_mask_7 k8_um_sized_commands = {
 };
 
 static const struct perfctr_unit_mask_4 k8_um_probe = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x00,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x0F,
       .nvalues = 4 },
     { { 0x01, "Probe miss" },
       { 0x02, "Probe hit" },
@@ -363,8 +391,8 @@ static const struct perfctr_unit_mask_4 k8_um_probe = {
 };
 
 static const struct perfctr_unit_mask_4 k8_um_ht = {
-    { .type = perfctr_um_type_exclusive,	/* XXX: bitmask? */
-      .default_value = 0x07,
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x0F,
       .nvalues = 4 },
     { { 0x01, "Command sent" },
       { 0x02, "Data sent" },
@@ -372,7 +400,7 @@ static const struct perfctr_unit_mask_4 k8_um_ht = {
       { 0x08, "Nop sent" } }
 };
 
-static const struct perfctr_event k8_events[] = {
+static const struct perfctr_event k8_common_events[] = {
     { 0x00, 0xF, UM(k8_um_fpu_ops), "DISPATCHED_FPU_OPS",
       "Dispatched FPU ops" },
     { 0x01, 0xF, NULL, "NO_FPU_OPS",
@@ -383,37 +411,42 @@ static const struct perfctr_event k8_events[] = {
       "Segment register load" },
     { 0x21, 0xF, NULL, "SELF_MODIFY_RESYNC",
       "Microarchitectural resync caused by self modifying code" },
-    { 0x22, 0xF, NULL, "SNOOP_RESYNC",	/* duplicate of 0x86 ? */
+    { 0x22, 0xF, NULL, "LS_RESYNC_BY_SNOOP",
+      /* similar to 0x86, but LS unit instead of IC unit */
       "Microarchitectural resync caused by snoop" },
     { 0x23, 0xF, NULL, "LS_BUFFER_FULL",
       "LS Buffer 2 Full" },
-    { 0x24, 0xF, NULL, "LOCKED_OP",
-      "Locked operation" },
     { 0x25, 0xF, NULL, "OP_LATE_CANCEL",
       "Microarchitectural late cancel of an operation" },
     { 0x26, 0xF, NULL, "CFLUSH_RETIRED",
       "Retired CFLUSH instructions" },
     { 0x27, 0xF, NULL, "CPUID_RETIRED",
       "Retired CPUID instructions" },
+    /* 0x40-0x47: from K7 official event set */
     { 0x48, 0xF, NULL, "ACCESS_CANCEL_LATE",
       "Microarchitectural late cancel of an access" },
     { 0x49, 0xF, NULL, "ACCESS_CANCEL_EARLY",
       "Microarchitectural early cancel of an access" },
     { 0x4A, 0xF, UM(k8_um_ecc), "ECC_BIT_ERR",
-      "One bit ECC error recorded by scrubber" },
+      "One bit ECC error recorded found by scrubber" },
     { 0x4B, 0xF, UM(k8_um_prefetch), "DISPATCHED_PRE_INSTRS",
       "Dispatched prefetch instructions" },
-    { 0x7D, 0xF, UM(k7_um_moesi), "BU_INT_L2_REQ",
+    { 0x76, 0xF, NULL, "CYCLES_PROCESSOR_IS_RUNNING", /* undocumented */
+      "Cycles processor is running (not in HLT or STPCLK)" },
+    { 0x7D, 0xF, UM(k8_um_int_L2_req), "BU_INT_L2_REQ",
       "Internal L2 request" },
-    { 0x7E, 0xF, UM(k7_um_moesi), "BU_FILL_REQ",
+    { 0x7E, 0xF, UM(k8_um_fill_req), "BU_FILL_REQ",
       "Fill request that missed in L2" },
-    { 0x7F, 0xF, UM(k7_um_moesi), "BU_FILL_L2",
+    { 0x7F, 0xF, UM(k8_um_fill_L2), "BU_FILL_L2",
       "Fill into L2" },
+    /* 0x80-0x81: from K7 official event set */
     { 0x82, 0xF, NULL, "IC_REFILL_FROM_L2",
       "Refill from L2" },
     { 0x83, 0xF, NULL, "IC_REFILL_FROM_SYS",
       "Refill from system" },
-    { 0x86, 0xF, NULL, "IC_RESYNC_BY_SNOOP",	/* duplicate of 0x22 ? */
+    /* 0x84-0x85: from K7 official event set */
+    { 0x86, 0xF, NULL, "IC_RESYNC_BY_SNOOP",
+      /* similar to 0x22, but IC unit instead of LS unit */
       "Microarchitectural resync caused by snoop" },
     { 0x87, 0xF, NULL, "IC_FETCH_STALL",
       "Instruction fetch stall" },
@@ -421,6 +454,7 @@ static const struct perfctr_event k8_events[] = {
       "Return stack hit" },
     { 0x89, 0xF, NULL, "IC_STACK_OVERFLOW",
       "Return stack overflow" },
+    /* 0xC0-0xC7: from K7 official event set */
     { 0xC8, 0xF, NULL, "RETIRED_NEAR_RETURNS",
       "Retired near returns" },
     { 0xC9, 0xF, NULL, "RETIRED_RETURNS_MISPREDICT",
@@ -431,10 +465,11 @@ static const struct perfctr_event k8_events[] = {
       "Retired FPU instructions" },
     { 0xCC, 0xF, UM(k8_um_fpu_fastpath), "RETIRED_FASTPATH_INSTRS",
       "Retired fastpath double op instructions" },
+    /* 0xCD-0xCF: from K7 official event set */
     { 0xD0, 0xF, NULL, "DECODER_EMPTY",
       "Nothing to dispatch (decoder empty)" },
     { 0xD1, 0xF, NULL, "DISPATCH_STALLS",
-      "Dispatch stalls" },
+      "Dispatch stalls (events 0xD2-0xDA combined)" },
     { 0xD2, 0xF, NULL, "DISPATCH_STALL_FROM_BRANCH_ABORT",
       "Dispatch stall from branch abort to retire" },
     { 0xD3, 0xF, NULL, "DISPATCH_STALL_SERIALIZATION",
@@ -468,15 +503,15 @@ static const struct perfctr_event k8_events[] = {
     { 0xE1, 0xF, NULL, "MEM_PAGE_TBL_OVERFLOW",
       "Memory controller page table overflow" },
     { 0xE2, 0xF, NULL, "DRAM_SLOTS_MISSED",
-      "Memory controller DRAM command slots missed in MemClks" },
+      "Memory controller DRAM command slots missed (in MemClks)" },
     { 0xE3, 0xF, UM(k8_um_turnaround), "MEM_TURNAROUND",
       "Memory controller turnaround" },
     { 0xE4, 0xF, UM(k8_um_saturation), "MEM_BYPASS_SAT",
-      "Memory controller bypass saturation" },
+      "Memory controller bypass counter saturation" },
     { 0xEB, 0xF, UM(k8_um_sized_commands), "SIZED_COMMANDS",
       "Sized commands" },
     { 0xEC, 0xF, UM(k8_um_probe), "PROBE_RESULT",
-      "Probe Result" },
+      "Probe result" },
     { 0xF6, 0xF, UM(k8_um_ht), "HYPERTRANSPORT_BUS0_WIDTH",
       "Hypertransport (tm) bus 0 bandwidth" },
     { 0xF7, 0xF, UM(k8_um_ht), "HYPERTRANSPORT_BUS1_WIDTH",
@@ -485,10 +520,63 @@ static const struct perfctr_event k8_events[] = {
       "Hypertransport (tm) bus 2 bandwidth" },
 };
 
-const struct perfctr_event_set perfctr_k8_event_set = {
+static const struct perfctr_event_set k8_common_event_set = {
     .cpu_type = PERFCTR_X86_AMD_K8,
     .event_prefix = "K8_",
     .include = &k7_official_event_set,
+    .nevents = ARRAY_SIZE(k8_common_events),
+    .events = k8_common_events,
+};
+
+static const struct perfctr_event k8_events[] = {
+     { 0x24, 0xF, NULL, "LOCKED_OP", /* unit mask changed in Rev. C */
+       "Locked operation" },
+};
+
+const struct perfctr_event_set perfctr_k8_event_set = {
+    .cpu_type = PERFCTR_X86_AMD_K8,
+    .event_prefix = "K8_",
+    .include = &k8_common_event_set,
     .nevents = ARRAY_SIZE(k8_events),
     .events = k8_events,
 };
+
+/*
+ * K8 Revision C.
+ * Current documentation does not describe how to distinguish
+ * Rev C parts from older parts.
+ */
+
+static const struct perfctr_unit_mask_3 k8c_um_locked_op = {
+     { .type = perfctr_um_type_bitmask,
+       .default_value = 0x01,
+       .nvalues = 3 },
+     { { 0x01, "Number of lock instructions executed" },
+       { 0x02, "Number of cycles spent in the lock request/grant stage" },
+       { 0x04, "Number of cycles a lock takes to complete once it is "
+	 "non-speculative and is the oldest load/store operation "
+	 "(non-speculative cycles in Ls2 entry 0)" } }
+};
+
+static const struct perfctr_unit_mask_2 k8c_um_lock_accesses = {
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x03,
+      .nvalues = 2 },
+    { { 0x01, "Number of dcache accesses by lock instructions" },
+      { 0x02, "Number of dcache misses by lock instructions" } }
+};
+
+static const struct perfctr_event k8c_events[] = {
+     { 0x24, 0xF, UM(k8c_um_locked_op), "LOCKED_OP", /* unit mask changed */
+       "Locked operation" },
+     { 0x4C, 0xF, UM(k8c_um_lock_accesses), "LOCK_ACCESSES",
+       "DCACHE accesses by locks" },
+};
+
+const struct perfctr_event_set perfctr_k8c_event_set = {
+    .cpu_type = PERFCTR_X86_AMD_K8,
+    .event_prefix = "K8C_",
+    .include = &k8_common_event_set,
+    .nevents = ARRAY_SIZE(k8c_events),
+    .events = k8c_events,
+};
