@@ -1,4 +1,4 @@
-/* $Id: event_set_p4.c,v 1.3 2003/05/15 22:39:17 mikpe Exp $
+/* $Id: event_set_p4.c,v 1.4 2003/06/15 21:15:00 mikpe Exp $
  * Performance counter event descriptions for Intel P4.
  *
  * Copyright (C) 2003  Mikael Pettersson
@@ -250,6 +250,50 @@ static const struct perfctr_unit_mask_3 p4_um_WC_Buffer = {
       { 0x04, "WCB_HITM_EVICT:store encountered a Hit Modified condition" } }
 };
 
+static const struct perfctr_unit_mask_6 p4_um_b2b_cycles = {
+    /* XXX: bits 1-6; no details documented yet */
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x7E,
+      .nvalues = 6 },
+    { { 0x02, "bit 1" },
+      { 0x04, "bit 2" },
+      { 0x08, "bit 3" },
+      { 0x10, "bit 4" },
+      { 0x20, "bit 5" },
+      { 0x40, "bit 6" } }
+};
+
+static const struct perfctr_unit_mask_3 p4_um_bnr = {
+    /* XXX: bits 0-2; no details documented yet */
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x07,
+      .nvalues = 3 },
+    { { 0x01, "bit 0" },
+      { 0x02, "bit 1" },
+      { 0x04, "bit 2" } }
+};
+
+static const struct perfctr_unit_mask_3 p4_um_snoop = {
+    /* XXX: bits 2, 6, and 7; no details documented yet */
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0xC4,
+      .nvalues = 3 },
+    { { 0x04, "bit 2" },
+      { 0x40, "bit 6" },
+      { 0x80, "bit 7" } }
+};
+
+static const struct perfctr_unit_mask_4 p4_um_response = {
+    /* XXX: bits 1, 2, 8, and 9; no details documented yet */
+    { .type = perfctr_um_type_bitmask,
+      .default_value = 0x306,
+      .nvalues = 4 },
+    { { 0x002, "bit 1" },
+      { 0x004, "bit 2" },
+      { 0x100, "bit 8" },
+      { 0x200, "bit 9" } }
+};
+
 static const struct perfctr_unit_mask_2 p4_um_nbogus_bogus = {
     { .type = perfctr_um_type_bitmask,
       .default_value = 0x01,
@@ -380,7 +424,7 @@ static const struct perfctr_event p4_events[] = {
       "x87 floating-point uops" },
     { 0x2E, FIRM_ESCR_0_1, UM(p4_um_x87_SIMD_moves_uop), "x87_SIMD_moves_uop",
       "x87 FPU, MMX, SSE, or SSE2 load, store, and move uops" },
-    { 0x05, FSB_ESCR_0_1, UM(p4_um_global_power_events), "global_power_events",
+    { 0x13, FSB_ESCR_0_1, UM(p4_um_global_power_events), "global_power_events",
       "time during which the processor is not stopped" },
     { 0x05, MS_ESCR_0_1, UM(p4_um_tc_ms_xfer), "tc_ms_xfer",
       "number of times uop delivery changed from TC to MS ROM" },
@@ -395,16 +439,16 @@ static const struct perfctr_event p4_events[] = {
       "stalls in the Allocator" },
     { 0x05, DAC_ESCR_0_1, UM(p4_um_WC_Buffer), "WC_Buffer",
       "write combining buffer operations" },
-    { 0x16, FSB_ESCR_0_1, NULL, "b2b_cycles",
+    { 0x16, FSB_ESCR_0_1, UM(p4_um_b2b_cycles), "b2b_cycles",
       /* XXX: may not be supported in all P4 models */
       "back-to-back bus cycles" },
-    { 0x04, FSB_ESCR_0_1, NULL, "bnr",
+    { 0x08, FSB_ESCR_0_1, UM(p4_um_bnr), "bnr",
       /* XXX: may not be supported in all P4 models */
       "bus not ready conditions" },
-    { 0x06, FSB_ESCR_0_1, NULL, "snoop",
+    { 0x06, FSB_ESCR_0_1, UM(p4_um_snoop), "snoop",
       /* XXX: may not be supported in all P4 models */
       "snoop hit modified bus traffic" },
-    { 0x04, FSB_ESCR_0_1, NULL, "response",
+    { 0x04, FSB_ESCR_0_1, UM(p4_um_response), "response",
       /* XXX: may not be supported in all P4 models */
       "different types of responses" },
     { 0x08, CRU_ESCR_2_3, UM(p4_um_nbogus_bogus), "front_end_event",
