@@ -1,4 +1,4 @@
-/* $Id: virtual.c,v 1.11 2003/03/10 22:07:58 mikpe Exp $
+/* $Id: virtual.c,v 1.13 2003/05/14 21:29:05 mikpe Exp $
  * Library interface to virtual per-process performance counters.
  *
  * Copyright (C) 1999-2003  Mikael Pettersson
@@ -6,6 +6,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 #include <unistd.h>
 #include <sys/mman.h>
@@ -87,6 +88,12 @@ int vperfctr_info(const struct vperfctr *vperfctr, struct perfctr_info *info)
 #define likely(x)	__builtin_expect((x),1)
 #define unlikely(x)	__builtin_expect((x),0)
 
+#if defined(__x86_64__)
+#define vperfctr_has_rdpmc(vperfctr)	(1)
+#else
+#define vperfctr_has_rdpmc(vperfctr)	((vperfctr)->have_rdpmc)
+#endif
+
 unsigned long long vperfctr_read_tsc(const struct vperfctr *self)
 {
     unsigned long long sum;
@@ -119,7 +126,7 @@ unsigned long long vperfctr_read_pmc(const struct vperfctr *self, unsigned i)
     kstate = self->kstate;
     cstatus = kstate->cpu_state.cstatus;
     /* gcc 3.0 generates crap code for likely(E1 && E2) :-( */
-    if( perfctr_cstatus_has_tsc(cstatus) && self->have_rdpmc ) {
+    if( perfctr_cstatus_has_tsc(cstatus) && vperfctr_has_rdpmc(self) ) {
 	 tsc0 = kstate->cpu_state.start.tsc;
     retry:
 	 rdpmcl(kstate->cpu_state.control.pmc_map[i], now);
@@ -137,10 +144,32 @@ unsigned long long vperfctr_read_pmc(const struct vperfctr *self, unsigned i)
     return kstate->cpu_state.sum.pmc[i];
 }
 
+static void vperfctr_read_ctrs_slow(const struct vperfctr *vperfctr,
+				    struct perfctr_sum_ctrs *sum)
+{
+    unsigned int tsc0, tsc1;
+    unsigned int cstatus, nrctrs;
+    volatile const struct vperfctr_state *kstate;
+    unsigned int i;
+
+    ioctl(vperfctr->fd, VPERFCTR_SAMPLE, NULL);
+    kstate = vperfctr->kstate;
+    cstatus = kstate->cpu_state.cstatus;
+    nrctrs = perfctr_cstatus_nrctrs(cstatus);
+    tsc1 = kstate->cpu_state.start.tsc;
+    do {
+	tsc0 = tsc1;
+	sum->tsc = kstate->cpu_state.sum.tsc;
+	for(i = 0; i < nrctrs; ++i)
+	    sum->pmc[i] = kstate->cpu_state.sum.pmc[i];
+	tsc1 = kstate->cpu_state.start.tsc;
+    } while( tsc1 != tsc0 );
+}
+
 void vperfctr_read_ctrs(const struct vperfctr *self,
 			struct perfctr_sum_ctrs *sum)
 {
-    unsigned int tsc0, tsc1, now;
+    unsigned int tsc0, now;
     unsigned int cstatus, nrctrs;
     volatile const struct vperfctr_state *kstate;
     int i;
@@ -151,7 +180,7 @@ void vperfctr_read_ctrs(const struct vperfctr *self,
     kstate = self->kstate;
     cstatus = kstate->cpu_state.cstatus;
     nrctrs = perfctr_cstatus_nrctrs(cstatus);
-    if( perfctr_cstatus_has_tsc(cstatus) && (!nrctrs || self->have_rdpmc) ) {
+    if( perfctr_cstatus_has_tsc(cstatus) && (!nrctrs || vperfctr_has_rdpmc(self)) ) {
     retry:
 	tsc0 = kstate->cpu_state.start.tsc;
 	rdtscl(now);
@@ -164,15 +193,7 @@ void vperfctr_read_ctrs(const struct vperfctr *self,
 	    return;
 	goto retry;
     }
-    ioctl(self->fd, VPERFCTR_SAMPLE, NULL);
-    tsc1 = kstate->cpu_state.start.tsc;
-    do {
-	tsc0 = tsc1;
-	sum->tsc = kstate->cpu_state.sum.tsc;
-	for(i = 0; i < nrctrs; ++i)
-	    sum->pmc[i] = kstate->cpu_state.sum.pmc[i];
-	tsc1 = kstate->cpu_state.start.tsc;
-    } while( tsc1 != tsc0 );
+    vperfctr_read_ctrs_slow(self, sum);
 }
 
 int vperfctr_read_state(const struct vperfctr *self, struct perfctr_sum_ctrs *sum,
@@ -247,13 +268,12 @@ struct rvperfctr *rvperfctr_open(int pid)
     char filename[64];
     struct rvperfctr *rvperfctr;
 
-    sprintf(filename, "/proc/%d/perfctr", pid);
+    snprintf(filename, sizeof filename, "/proc/%d/perfctr", pid);
     rvperfctr = malloc(sizeof(*rvperfctr));
     if( rvperfctr ) {
 	if( vperfctr_open_name(filename, &rvperfctr->vperfctr) == 0 ) {
 	    rvperfctr->pid = pid;
-	    /* clear have_rdpmc to ensure read_ctrs() avoids fast path */
-	    rvperfctr->vperfctr.have_rdpmc = 0;
+	    return rvperfctr;
 	}
 	free(rvperfctr);
     }
@@ -273,7 +293,7 @@ int rvperfctr_info(const struct rvperfctr *rvperfctr, struct perfctr_info *info)
 void rvperfctr_read_ctrs(const struct rvperfctr *rvperfctr,
 			 struct perfctr_sum_ctrs *sum)
 {
-    return vperfctr_read_ctrs(&rvperfctr->vperfctr, sum);
+    return vperfctr_read_ctrs_slow(&rvperfctr->vperfctr, sum);
 }
 
 int rvperfctr_read_state(const struct rvperfctr *rvperfctr,
